@@ -29,6 +29,7 @@
 
 #include "webp/decode.h"
 #include "webp/encode.h"
+#include "webp/demux.h"
 
 /* Used for xv to hand off save info to our 'library' */
 static char *filename;
@@ -476,6 +477,7 @@ int LoadWEBP(char *fname, PICINFO *pinfo)
   uint8_t               *raw_data, *rgba, alpha;
   WebPBitstreamFeatures features;
   VP8StatusCode         status;
+  uint32_t				format_flags;
 
   /* open the file */
   fp = xv_fopen(fname,"r");
@@ -644,6 +646,52 @@ int LoadWEBP(char *fname, PICINFO *pinfo)
     pinfo->pic[i*3 + 2] = *(rgba + i*4 + 2) * alpha/255;
   }
 
+#ifdef HAVE_EXIF
+  /* If this image contains an EXIF orientation flag,
+  ** read it and set it appropriately in pinfo.
+  */
+
+  WebPDemuxer* demux;
+  WebPChunkIterator chunk_iter;
+  unsigned int exif_length;
+  byte *exif_data_src;
+
+  demux = WebPDemux((WebPData *)&raw_data);
+  format_flags = WebPDemuxGetI(demux, WEBP_FF_FORMAT_FLAGS);
+
+  if (format_flags & EXIF_FLAG)
+  {
+    WebPDemuxGetChunk(demux, "EXIF", 1, &chunk_iter);
+
+    /* The returned EXIF chunk contains the raw TIFF
+	** image metadata. However, libexif expects it
+	** to be prefixed with a "Exif\0\0" marker and
+	** won't read it unless that's there. So put it
+	** there and pretend that's what we read from
+	** the WebP chunk.
+	*/
+
+    exif_length = chunk_iter.chunk.size + 6;
+
+	if ((exif_data_src = malloc(exif_length)) == NULL)
+	{
+      FatalError("malloc failure in LoadWEBP");
+	}
+	
+	memcpy(exif_data_src, "Exif", 4);
+	memset(exif_data_src + 4, 0, 2);
+	memcpy(exif_data_src + 6, chunk_iter.chunk.bytes, chunk_iter.chunk.size);
+
+	pinfo->orientation = get_exif_orientation(exif_data_src, chunk_iter.chunk.size + 6);
+
+	free(exif_data_src);
+  }
+
+  WebPDemuxReleaseChunkIterator(&chunk_iter);
+  WebPDemuxDelete(demux);
+
+#endif /* HAVE_EXIF */
+
   free(raw_data);
   WebPFree(rgba);
   return 1;
